from .datadome import TakionAPIDatadome
from .incapsula import TakionAPIUtmvc, TakionAPIReese84

from .exceptions import (
    TakionAPIException,
    IpBanException,
    BadResponseException
)

__all__ = [
    "TakionAPIDatadome",
    "TakionAPIUtmvc",
    "TakionAPIReese84",
    "TakionAPIException",
    "IpBanException",
    "BadResponseException"
]

__version__ = "0.1.3"



class TakionAPIException(Exception):
    """
    # Takion API Exception
    Base exception for all exceptions raised by the Takion API
    """
    pass

class IpBanException(TakionAPIException):
    """
    # IP Ban Exception
    Exception raised when the IP is banned
    """
    pass

class BadResponseException(TakionAPIException):
    """
    # Bad Response Exception
    Exception raised when the response is invalid
    """
    pass


from requests import get
from requests.models import Response
from typing import Optional, Dict

from ..exceptions import BadResponseException
from ..models import CookieRequest

class TakionAPIReese84:
    BASE_URL = "https://takionapi.tech/incapsula/payload/{}?api_key={}"
    api_key: str

    @staticmethod
    def is_challenge(response: Response) -> bool:
        """
        # Reese84 cookie needed?
        ## Check if the response has been blocked by Incapsula
        This is a very basic check, should **never** be used in production, since each website
        uses a different method to return an incapsula challenge page.

        ### Parameters
        - `response`: The response object to check

        ### Returns
        - `bool`: True if the response is a Incapsula challenge, False otherwise
        """
        return response.status_code in [403, 401]

    def __init__(
        self,
        api_key: str,
    ) -> None:
        '''
        # Takion API Reese84
        ## Incapsula Reese84 API wrapper for Takion
        This class is a wrapper for the Incapsula API, it can be used to solve reese84 challenge.
        To get your takion API key please check the [Takion API](https://takionapi.tech/) website.

        ### Parameters
        - `api_key`: The API key to use

        ### Example Usage
        ```py
        from requests import Session
        from takion_api import TakionAPIReese84

        session = Session()
        takion_api = TakionAPIReese84(
            api_key="TAKION_API_XXXXXXXXXX"
        )
        
        protected_url = "https://www.ticketmaster.com/event/1C005E959B003CA9"
        headers = {
            "Host": "www.ticketmaster.com",
            "sec-ch-ua-platform": "\"Windows\"",
            "sec-ch-ua-mobile": "?0",
            "upgrade-insecure-requests": "1",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36",
            "sec-fetch-mode": "navigate",
            "sec-fetch-dest": "document",
            "sec-ch-ua": '"Chromium";v="110", "Not A(Brand";v="24", "Google Chrome";v="110"',
            "sec-fetch-site": "same-site",
            "sec-fetch-user": "?1",
            "accept-language": "en-US,en;q=0.9"
        }
        
        def send_request_with_solving() -> None:
            # Create a new session
            session = Session()
            print("Loading page with solved reese84...")

            # Generate payload
            data = takion_api.solve_challenge("www.ticketmaster.com", headers["user-agent"])
            
            # Send challenge
            res = session.post(
                data["url"], 
                data=data["payload"], 
                headers=data["headers"]
            ).json()
            
            # Set cookie
            session.cookies.set("reese84", res['token'])
            print(f"Got cookie: {res['token'][:15]}...{res['token'][-15:]}")

            # Sending request to protected url 
            res = session.get(protected_url, headers=headers)
            print(f"Response {'not ' if not TakionAPIReese84.is_challenge(res) else ''}blocked (Status code {res.status_code})")

        def send_request_without_solving() -> None:
            # Create a new session
            session = Session()
            print("Loading page without solving the challenge...")

            # Sending request to protected url
            res = session.get(protected_url, headers=headers)
            print(f"Response {'not ' if not TakionAPIReese84.is_challenge(res) else ''}blocked (Status code {res.status_code})")

        if __name__ == "__main__":
            send_request_without_solving() # Will return 403 or 401
            print("-----------------------------------")
            send_request_with_solving() # Will return 404, old event
        '''
        self.api_key = api_key
        pass
    
    def solve_challenge(
        self,
        website_domain: str,
        user_agent: Optional[str]="Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36",
    ) -> Dict[str, CookieRequest]:
        '''
        # Solve the challenge
        Solve the challenge and return the payload, headers and url for the cookie generation request

        ### Parameters
        - `website_domain`: The website domain to use for the payload generation
        - `user_agent`: The User-Agent used to request the challenge page

        ### Returns
        - `Dict[str, CookieRequest]`: The payload, headers and url for the cookie generation request

        ### Raises
        - `BadResponseException`: If the challenge could not be solved
        '''
        try:
            res = get(
                self.BASE_URL.format(
                    website_domain,
                    self.api_key,
                ),
                headers={
                    "User-Agent": user_agent,
                }
            ).json()
        except:
            raise BadResponseException("Could not solve challenge")
        if (error := res.get("error")):
            raise BadResponseException(error)
        return res


from requests import post, get
from requests.models import Response
from requests.cookies import RequestsCookieJar
from re import search
from typing import Optional, Union

from ..exceptions import BadResponseException

class TakionAPIUtmvc:
    BASE_URL = "https://takionapi.tech/incapsula/utmvc?api_key={}"
    api_key: str

    @staticmethod
    def is_challenge(response: Response) -> Union[bool, str]:
        """
        # Utmvc cookie needed?
        ## Check if the response has the utmvc script in it
        This is a very basic check, it only checks if the html page returned by the server contains the utmvc script.
        This method should be edited based on the website you are trying to access.

        ### Parameters
        - `response`: The response object to check

        ### Returns
        - `Union[bool, str]`: The utmvc script if the response is a Utmvc challenge, False otherwise
        """
        pattern = r'src="(/_Incapsula_Resource\?[^"]+)"'
        match = search(pattern, response.text)
        return f"https://{response.url.split('/')[2]}/{match.group(1)}" if match else False

    def __init__(
        self,
        api_key: str,
    ) -> None:
        '''
        # Takion API Utmvc
        ## Incapsula Utmvc API wrapper for Takion
        This class is a wrapper for the Incapsula API, it can be used to solve utmvc challenge.
        To get your takion API key please check the [Takion API](https://takionapi.tech/) website.

        ### Parameters
        - `api_key`: The API key to use

        ### Example Usage
        ```py
        from requests import Session
        from takion_api import TakionAPIUtmvc

        session = Session()
        takion_api = TakionAPIUtmvc(
            api_key="TAKION_API_XXXXXXXXXX"
        )
        url = "https://tickets.rolandgarros.com/fr/"
        headers = {...}
        response = session.get(url, headers=headers)
        if not takion_api.is_challenge(response):
            print("Page loaded successfully")
        else:
            print("Found challenge, solving...")
            # Solve the challenge
            utmvc = takion_api.solve_challenge(
                response,
                cookies=session.cookies # Optional but recommended
            )
            # Set the utmvc cookie
            session.cookies.set("___utmvc", utmvc)
            print(f"Got cookie: {utmvc[:15]}...{utmvc[-15:]}")
            # Now we send the original request again
            response = session.get(url, headers=headers)
            print(f"Challenge {'' if takion_api.is_challenge(response) else 'not '}found using cookie")
        '''
        self.api_key = api_key
        pass

    def get_challenge_url(
        self,
        user_agent: Optional[str]="Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36",
    ) -> str:
        '''
        # Load the challenge script
        
        ### Parameters
        - `user_agent`: The User-Agent used to request the challenge page

        ### Returns
        - `str`: The challenge URL

        ### Raises
        - `BadResponseException`: If the challenge URL could not be parsed from the response
        '''
        try:
            return get(
                self.challenge_url,
                headers={
                    "User-Agent": user_agent,
                },
            ).text
        except:
            raise BadResponseException("Could not parse challenge data from response")
    
    def solve_challenge(
        self,
        response: Response,
        user_agent: Optional[str]="Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36",
        cookies: Optional[RequestsCookieJar]=None,
    ) -> str:
        '''
        # Solve the challenge
        Solve the challenge and return the payload, headers and url for the cookie generation request

        ### Parameters
        - `response`: The response of the challenge page (the page returned by get_challenge_url)
        - `user_agent`: The User-Agent used to request the challenge page
        - `cookies`: The cookies to send with the request

        ### Returns
        - `str`: The utmvc cookie

        ### Raises
        - `BadResponseException`: If the challenge could not be solved
        '''
        self.challenge_url = TakionAPIUtmvc.is_challenge(response)
        if not self.challenge_url:
            return False
        
        content = self.get_challenge_url(user_agent)
        try:
            res = post(
                self.BASE_URL.format(
                    self.api_key,
                ),
                json={
                    "content": content,
                    "cookies": [[name, value] for name, value in cookies.items()]
                },
                headers={
                    "User-Agent": user_agent,
                }
            ).json()
        except:
            raise BadResponseException("Could not solve challenge")
        if (error := res.get("error")):
            raise BadResponseException(error)
        return res.get("___utmvc")


from setuptools import setup, find_packages

setup(
    name='takion_api',
    version='0.1.31',
    packages=find_packages(),
    install_requires=[
        line.strip() for line in open("requirements.txt", "r").readlines()
    ],
    author='glizzykingdreko',
    author_email='glizzykingdreko@protonmail.com',
    description='Official Python wrapper for TakionAPI services, designed for efficient automation and bypass of advanced antibot systems.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/Takion-API-Services/takion-api',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    keywords=[
        "takion-api",
        "takion",
        "datadome",
        "antibot",
        "bypass",
    ],
    python_requires='>=3.6',
)


from requests import post
from requests.models import Response
from typing import Optional, Dict

from ..exceptions import BadResponseException, IpBanException
from ..models import CookieRequest

class TakionAPIDatadome:
    BASE_URL = "https://takionapi.tech/datadome/{}?api_key={}"
    api_key: str
    challenge_type: Optional[str]

    @staticmethod
    def is_challenge(response: Response) -> bool:
        """
        # Got Flagged by Datadome?
        ## Check if the response is a Datadome challenge
        This is a very basic check, it only checks if the server is Datadome protected,
        response status code is 403 or the page contains the Datadome domains.
        This method should be edited based on the website you are trying to access.

        ### Parameters
        - `response`: The response object to check

        ### Returns
        - `bool`: True if the response is a Datadome challenge, False otherwise
        """
        if not response.headers.get("Server") == "DataDome":
            # Server is not Datadome protected
            return False
        if response.status_code == 403:
            # Server is Datadome protected 
            # and we has been blocked
            return True
        # Check if the page contains the Datadome domains
        return "geo.captcha-delivery.com" in response.text or \
            "interstitial.captcha-delivery.com" in response.text

    @staticmethod
    def extract_cookie(response: Response) -> str:
        '''
        ## Extract Cookie
        Extract the Datadome cookie from the response headers
        
        ### Parameters 
        - `response`: The response object to extract the cookie from
        
        ### Returns
        - `str`: The Datadome cookie
        
        ### Raises
        - `BadResponseException`: If the cookie could not be extracted from the response
        '''
        try:
            return response.json()["cookie"].split(";")[0].split("=")[1]
        except:
            raise BadResponseException("Could not extract cookie from response")

    def __init__(
        self,
        api_key: str,
    ) -> None:
        '''
        # Takion API Datadome
        ## Datadome API wrapper for Takion
        This class is a wrapper for the Datadome API, it can be used to solve Datadome challenges.
        To get your takion API key please check the [Takion API](https://takionapi.tech/) website.

        ### Parameters
        - `api_key`: The API key to use

        ### Example Usage
        ```py
        from requests import Session
        from takion_api import TakionAPIDatadome

        session = Session()
        takion_api = TakionAPIDatadome(
            api_key="TAKION_API_XXXXXXXXXX"
        )
        url = "https://www.footlocker.pt/en/product/~/314206535404.html"
        headers = {...}
        response = session.get(url, headers=headers)
        if not takion_api.is_challenge(response):
            print("Page loaded successfully")
        else:
            print("Solving challenge...")
            # Load the challenge url
            challenge_url = takion_api.get_challenge_url(
                url,
                response,
                session.cookies.get_dict().get("datadome")
            )
            # Load the challenge page
            challenge_response = session.get(challenge_url, headers=headers)
            # Solve the challenge
            data = takion_api.solve_challenge(
                challenge_response
            )
            # Send the cookie generation request
            cookie_response = session.post(
                data["url"], 
                data=data["payload"], 
                headers=data["headers"]
            )
            cookie = TakionAPIDatadome.extract_cookie(cookie_response)
            session.cookies.set("datadome", cookie)
            print(f"Got cookie: {cookie[:15]}...{cookie[-15:]}")
            # Now we send the original request again
            response = session.get(url, headers=headers)
            print(f"Got a response with status code {response.status_code} with the cookie")
        '''
        self.api_key = api_key
        self.challenge_type = None
        pass

    def get_challenge_url(
        self,
        url: str,
        response: Response,
        cookie: str,
        user_agent: Optional[str]="Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36",
    ) -> str:
        '''
        # Parse, build and return the challenge URL
        
        ### Parameters
        - `url`: The URL of the page that returned the challenge
        - `response`: The response of the challenge page (the page returned by get_challenge_url)
        - `cookie`: The Datadome cookie setted on the previous request
        - `user_agent`: The User-Agent used to request the challenge page

        ### Returns
        - `str`: The challenge URL

        ### Raises
        - `BadResponseException`: If the challenge URL could not be parsed from the response
        '''
        try:
            res = post(
                self.BASE_URL.format(
                    "build-url",
                    self.api_key,
                ),
                json={
                    "html": response.text,
                    "cid": cookie,
                    "referrer": url,
                },
                headers={
                    "User-Agent": user_agent,
                },
            ).json()
        except:
            raise BadResponseException("Could not parse challenge URL from response")
        if not (url := res.get("url")):
            raise BadResponseException(res.get('error'))
        self.challenge_type = res.get("challenge_type")
        return url
    
    def solve_challenge(
        self,
        response: Response,
        user_agent: Optional[str]="Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36",
        challenge_type: Optional[str]=None,
    ) -> Dict[str, CookieRequest]:
        '''
        # Solve the challenge
        Solve the challenge and return the payload, headers and url for the cookie generation request

        ### Parameters
        - `response`: The response of the challenge page (the page returned by get_challenge_url)
        - `user_agent`: The User-Agent used to request the challenge page
        - `challenge_type`: The challenge type to use (optional, can be `interstitial` or `geetest`)

        ### Returns
        - `Dict[str, CookieRequest]`: The payload, headers and url for the cookie generation request

        ### Raises
        - `BadResponseException`: If the challenge could not be solved
        - `ValueError`: If the challenge type is not set and the challenge_type parameter is not set
        - `IpBanException`: If the IP is banned

        ### Notes
        - The challenge type is automatically set by the get_challenge_url method
        - The challenge type can be forced by the challenge_type parameter
        '''
        if not self.challenge_type and challenge_type not in ["interstitial", "geetest"]:
            raise ValueError("Challenge type not set")
        elif not self.challenge_type and challenge_type in ["interstitial", "geetest"]:
            self.challenge_type = challenge_type
        
        try:
            res = post(
                self.BASE_URL.format(
                    self.challenge_type,
                    self.api_key,
                ),
                json={
                    "html": response.text,
                },
                headers={
                    "User-Agent": user_agent,
                }
            ).json()
        except:
            raise BadResponseException("Could not solve challenge")
        if (error := res.get("error")):
            if error == 'Ip banned': raise IpBanException(error)
            raise BadResponseException(error)
        return res